'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getAllPaths = exports.getNestedPaths = exports.log = undefined;

var _isArray = require('lodash/isArray');

var _isArray2 = _interopRequireDefault(_isArray);

var _isUndefined = require('lodash/isUndefined');

var _isUndefined2 = _interopRequireDefault(_isUndefined);

var _flattenDeep = require('lodash/flattenDeep');

var _flattenDeep2 = _interopRequireDefault(_flattenDeep);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _package = require('../package.json');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * A simple debug logger
 */
var log = exports.log = (0, _debug2.default)(_package.name);

/**
 * This is not a very sophisticated checking method. Assuming we already know
 * this is either a Route or an IndexRoute under what cases would this break?
 */
var hasNoComponent = function hasNoComponent(route) {
  return (0, _isUndefined2.default)(route.props.path) || (0, _isUndefined2.default)(route.props.component);
};

/**
 * NOTE: We could likely use createRoutes to our advantage here. It may simplify
 * the code we currently use to recurse over the virtual dom tree:
 *
 * import { createRoutes } from 'react-router';
 * console.log(createRoutes(routes)); =>
 * [ { path: '/',
 *    component: [Function: Layout],
 *    childRoutes: [ [Object], [Object] ] } ]
 *
 * Ex:
 * const routes = (
 *   <Route component={App} path='/'>
 *     <Route component={About} path='/about' />
 *   </Route>
 * );
 *
 * getAllPaths(routes); => ['/', '/about]
 */
var getNestedPaths = exports.getNestedPaths = function getNestedPaths(route) {
  var prefix = arguments.length <= 1 || arguments[1] === undefined ? '' : arguments[1];

  if (!route) return [];

  if ((0, _isArray2.default)(route)) return route.map(function (x) {
    return getNestedPaths(x, prefix);
  });

  // Some routes such as redirects or index routes do not have a component. Skip
  // them.
  if (hasNoComponent(route)) return [];

  var path = prefix + route.props.path;
  var nextPrefix = path === '/' ? path : path + '/';
  return [path].concat(getNestedPaths(route.props.children, nextPrefix));
};

var getAllPaths = exports.getAllPaths = function getAllPaths(routes) {
  return (0, _flattenDeep2.default)(getNestedPaths(routes));
};

exports.default = getAllPaths;